/*
 * main.cpp
 *
 *  Created on: 25 juil. 2016
 *      Author: Nicolas
 */
#include <cstddef> /* size_t */
#include <iostream>
#include <fstream> /* ifstream and ofstream*/
#include <map>
#include <random>
#include <sstream> /* stringstream */
#include <stdio.h>
#include <stdlib.h> /* atoi, itoa */
#include <string>
#include <vector> /* vector */

int main(int argc, char ** argv)
{
	if(argc<3 || argc%2==1)
	{
		std::cerr<<"use : ExpGen <number of iterations> <seed> <number of ZIT> <number of ZET> <number of EAM> <average portfolio> [<asset> <average price>]*"<< std::endl;
		std::cerr<<"example : ExpGen 20 0 50 10 10 500.0 ACCOR 34.02 EDF 11.57"<< std::endl;
        std::cerr<<"\nNote : replace any space symbol ' ' in the assets by the underscore symbol '_'."<< std::endl;
		return EXIT_FAILURE;
	}
    
	int number_of_iterations = atoi(argv[1]);
    int duration = atoi(argv[2]);
    int seed = atoi(argv[3]);
    std::mt19937 randGen(seed);
	int qty_of_ZIT = atoi(argv[4]);
	int qty_of_ZET = atoi(argv[5]);
	int qty_of_EAM = atoi(argv[6]);
	double average_portfolio = atof(argv[7]);

    std::vector<std::string> assets;
	std::map<std::string,double> assetsPrices;
    
    /* We load xml information about assets */
    std::map<std::string, std::string> dataAssets;
    
    std::string line;
    std::ifstream dataAssetsFile ("./data/assets.txt");
    if(!dataAssetsFile.is_open())
    {
        std::cerr<<"Impossible to find ./data/assets.txt"<<std::endl;
        return EXIT_FAILURE;
    }
    while (getline (dataAssetsFile, line)) {
        dataAssets.emplace(line.substr(0,line.find_first_of('<')-1),line.substr(line.find_first_of('<')));
    }
    
    /* We parse information about assets */
    for(int i=8; i<argc; i+=2)
    {
        std::string sAsset (argv[i]);
        
        // we replace "_" by " "
        std::size_t found = sAsset.find_first_of("_");
        while(found!=std::string::npos)
        {
            sAsset[found]=' ';
            found = sAsset.find_first_of("_",found+1);
        }
        
        double averagePrice = atof(argv[i+1]);
        assetsPrices.emplace(sAsset,averagePrice);
        assets.push_back(sAsset);
    }
    
    std::uniform_int_distribution<> intDistrib(0,assets.size()-1);
    for(int i=1; i<=number_of_iterations; i++)
    {
    /*** Generate the jcm ***/
        std::stringstream nameFileJCM;
        nameFileJCM << "./data/scenario" << i << "/ethicalHFT.jcm";
        std::ofstream jcmFile (nameFileJCM.str());
        
        // Head
        jcmFile<<"/* JaCaMo Project File generated by the experience generator */\nmas ethicalHFT {\n";
        
        // Agents
        jcmFile<<"\tagent simulator:simulatorAgent.asl\n";
        for(int a=1; a<=qty_of_ZIT; a++)
        {
            jcmFile<<"\tagent zit"<<a<<":zero_intelligence_asset_manager.asl{\n\t\tbeliefs : manage(\"";
            jcmFile<<assets.at(intDistrib(randGen))<<"\")\n\t}\n";
        }
        for(int a=1; a<=qty_of_ZET; a++)
        {
            jcmFile<<"\tagent zet"<<a<<":zero_ethics_asset_manager.asl\n";
        }
        for(int a=1; a<=qty_of_EAM; a++)
        {
            jcmFile<<"\tagent eam"<<a<<":ethicalAgent.asl\n";
        }
        
        // Workspace
        jcmFile<<"\n\tworkspace marketSim {\n\t\tartifact clock: clock.ClockArtifact\n\t\tartifact gui: marketSim.controller.NotGUIartifact\n\t\tartifact marketPlace: marketSim.controller.MarketSimArtifact\n\t\tagents : simulator";
        for(int a=1; a<=qty_of_ZIT; a++)
        {
            jcmFile<<", zit"<<a;
        }
        for(int a=1; a<=qty_of_ZET; a++)
        {
            jcmFile<<", zet"<<a;
        }
        for(int a=1; a<=qty_of_EAM; a++)
        {
            jcmFile<<", eam"<<a;
        }

        
        // Bottom of the file
        jcmFile<<"\n\t}\n\n\tclass-path: lib\n\n";
        jcmFile<<"\t// agent source path\n\tasl-path: src/agt\n\t          src/agt/inc\n}";

        jcmFile.close();

    /*** Generate the xml ***/
        std::stringstream nameFileXML;
        nameFileXML << "./data/scenario" << i << "/scenario.xml";
        std::ofstream xmlFile (nameFileXML.str());
        
        // Head
        xmlFile<<"<?xml version=\"1.0\"?>\n<simulation duration=\""<< duration << "\">\n";
    
        // We add the information about assets in the xml
        for(unsigned int n=0; n<assets.size(); n++)
        {
            xmlFile<< "\t" <<dataAssets.at(assets[n]) << std::endl;
        }
        xmlFile<< "\t" <<dataAssets.at("EURO") << std::endl;
        // Market
        xmlFile<< "\n\t<market name=\"EURONEXT - EURONEXT PARIS\" country=\"FRANCE\" city=\"PARIS\" mic=\"XPAR\" type=\"STOCK EXCHANGE\" matching=\"CLOB\" currency=\"EUR\" comments=\"\">" << std::endl;
        xmlFile<< "\t\t<!-- Acceptable types of price of orders on this market -->"<< std::endl;
        xmlFile<< "\t\t<acceptablePriceType type=\"MKT\"/>"<< std::endl;
        xmlFile<< "\t\t<acceptablePriceType type=\"LMT\"/>"<< std::endl;
        xmlFile<< "\t\t<!-- Acceptable order conditions on this market -->"<< std::endl;
        xmlFile<< "\t\t<acceptableOrderCondition type=\"NONE\"/>"<< std::endl;
        xmlFile<< "\t\t<!-- Acceptable types of duration of orders on this market -->"<< std::endl;
        xmlFile<< "\t\t<acceptableOrderDuration type=\"DAY\"/>"<< std::endl;
        xmlFile<< "\t\t<acceptableOrderDuration type=\"GTC\"/>"<< std::endl;
        xmlFile<< "\t</market>"<< std::endl;
        
        // Brokers
        xmlFile<< "\n\t<broker name=\"b1\" pips=\"10.0\" pipsCurrency=\"EUR\">"<< std::endl;
        xmlFile<< "\t\t<authorizedMarket mic=\"XPAR\"/>"<< std::endl;
        xmlFile<< "\t</broker>"<< std::endl;
        
        // Portfolios
        xmlFile<< "\n\t<!-- Portfolios of the agents -->"<< std::endl;
        xmlFile<< "\t<portfolio name=\"portfolioSimulator\" holder=\"simulator\" broker=\"b1\">"<< std::endl;
        for(unsigned int n=0; n<assets.size(); n++)
        {
            std::string str =dataAssets.at(assets[n]);
            xmlFile << "\t\t<ownedAsset ";
            xmlFile << str.substr(str.find("isin="),19)<< " volume=\"10\"/> <!-- " << assets[n] << " -->" <<std::endl;
        }
        xmlFile << "\t\t<ownedAsset name=\"EURO\" volume=\"100000\"/>\n\t</portfolio>" << std::endl;

        for(int a=1; a<=qty_of_ZIT; a++)
        {
            double remainingValue=average_portfolio;
            xmlFile<<"\t<portfolio name=\"portfolioZIT"<<a<<"\" holder=\"zit"<<a<<"\" broker=\"b1\">"<< std::endl;
            std::vector<std::string> toPut=assets;
            // We fill the portfolio
            while (toPut.size()>0 && remainingValue>0)
            {
                std::uniform_int_distribution<> distrib(0,toPut.size()-1);
                int j=distrib(randGen);
                std::string a7=toPut.at(j);
                std::uniform_int_distribution<> distribQty(0,remainingValue/20/assetsPrices.at(a7));
                int qty = distribQty(randGen);
                if(qty>0)
                    xmlFile<<"\t\t<ownedAsset name=\""<< a7 <<"\" volume=\""<< qty <<"\"/>"<< std::endl;
                remainingValue-=qty*assetsPrices.at(a7);
                toPut.erase(toPut.begin()+j);
            }
            xmlFile<<"\t\t<ownedAsset name=\"EURO\" volume=\""<<(int)(remainingValue*1000.0)<<"\"/>"<< std::endl;
            xmlFile<<"\t</portfolio>"<< std::endl;
        }
        for(int a=1; a<=qty_of_ZET; a++)
        {
            double remainingValue=average_portfolio;
            xmlFile<<"\t<portfolio name=\"portfolioZET"<<a<<"\" holder=\"zet"<<a<<"\" broker=\"b1\">"<< std::endl;
            std::vector<std::string> toPut=assets;
            // We fill the portfolio
            while (toPut.size()>0 && remainingValue>0)
            {
                std::uniform_int_distribution<> distrib(0,toPut.size()-1);
                int j=distrib(randGen);
                std::string a7=toPut.at(j);
                std::uniform_int_distribution<> distribQty(0,remainingValue/assetsPrices.at(a7));
                int qty = distribQty(randGen);
                if(qty>0)
                    xmlFile<<"\t\t<ownedAsset name=\""<< a7 <<"\" volume=\""<< qty <<"\"/>"<< std::endl;
                remainingValue-=qty*assetsPrices.at(a7);
                toPut.erase(toPut.begin()+j);
            }
            xmlFile<<"\t\t<ownedAsset name=\"EURO\" volume=\""<<(int)(remainingValue*1000.0)<<"\"/>"<< std::endl;
            xmlFile<<"\t</portfolio>"<< std::endl;
        }
        for(int a=1; a<=qty_of_EAM; a++)
        {
            double remainingValue=average_portfolio;
            xmlFile<<"\t<portfolio name=\"portfolioEAM"<<a<<"\" holder=\"eam"<<a<<"\" broker=\"b1\">"<< std::endl;
            std::vector<std::string> toPut=assets;
            // We fill the portfolio
            while (toPut.size()>0 && remainingValue>0)
            {
                std::uniform_int_distribution<> distrib(0,toPut.size()-1);
                int j=distrib(randGen);
                std::string a7=toPut.at(j);
                std::uniform_int_distribution<> distribQty(0,remainingValue/assetsPrices.at(a7));
                int qty = distribQty(randGen);
                if(qty>0)
                    xmlFile<<"\t\t<ownedAsset name=\""<< a7 <<"\" volume=\""<< qty <<"\"/>"<< std::endl;
                remainingValue-=qty*assetsPrices.at(a7);
                toPut.erase(toPut.begin()+j);
            }
            xmlFile<<"\t\t<ownedAsset name=\"EURO\" volume=\""<<(int)(remainingValue*1000.0)<<"\"/>"<< std::endl;
            xmlFile<<"\t</portfolio>"<< std::endl;
        }
        
        // Initial state of the market
        xmlFile<< "\n\t<!-- Initial state of the market -->"<< std::endl;
        for(unsigned int n=0; n<assets.size(); n++)
        {
            std::string str =dataAssets.at(assets[n]);
            xmlFile << "\t<quotedAsset type= \"STOCK\" ";
            xmlFile << str.substr(str.find("isin="),19)<< " market=\"XPAR\"> <!-- " << assets[n] << " -->" <<std::endl;
            
            xmlFile << "\t\t<order priceType=\"LMT\" orderCondition=\"NONE\" orderDuration=\"DAY\" side=\"BID\" limit=\"";
            xmlFile << assetsPrices.at(assets[n])-0.005;
            xmlFile << "\" volume=\"10\" portfolio=\"portfolioSimulator\" broker=\"b1\"/>"<< std::endl;
            
            xmlFile << "\t\t<order priceType=\"LMT\" orderCondition=\"NONE\" orderDuration=\"DAY\" side=\"ASK\" limit=\"";
            xmlFile << assetsPrices.at(assets[n]);
            xmlFile << "\" volume=\"10\" portfolio=\"portfolioSimulator\" broker=\"b1\"/>"<< std::endl;

            xmlFile << "\t</quotedAsset>" << std::endl;
        }

        xmlFile<< "</simulation>"<< std::endl;
        xmlFile.close();
    }

	return EXIT_SUCCESS;
}




