// Agent rightness_process in project ethicalHFT

//*** Ethical principles ***//

// Perfect action : a moral, possible, and desired one
ethPrinciple("perfectAct",Action):-
	possible_eval(Action, possible) &
	desire_eval(Action,desired) &
	not desire_eval(Action,undesired) &
	moral_eval(Action,_,moral) &
	not moral_eval(Action,_,immoral).

ethPrinciple("desireNR",Action):-
	possible_eval(Action, possible) &
	desire_eval(Action,desired) &
	not desire_eval(Action,undesired) &
	not moral_eval(Action,_,immoral).
	
ethPrinciple("dutyNR",Action):-
	possible_eval(Action, possible) &
	not desire_eval(Action,undesired) &
	moral_eval(Action,_,moral) &
	not moral_eval(Action,_,immoral).

// Aristotelian ethics inspired from
//	- [1] J.G. Ganascia, "Modelling ethical rules of lying with Answer Set Programming"
//  - [2] J.G. Ganascia, "Ethical System Formalization using Non-Monotonic Logics"
ethPrinciple("aristotelian",Action):-
	possible_eval(Action, possible) &
	aristotelian_just(Action).

// These Rules are directly translated from [1]
aristotelian_just(Action):-
	aristotelian_worst_consequence(Action, Consequence)
	& aristotelian_worst_consequence(OtherAction, ConsequenceOtherAction)
	& aristotelian_worse(ConsequenceOtherAction, Consequence)
	& not aristotelian_unjust(Action).
	
aristotelian_unjust(Action):-
	aristotelian_worst_consequence(Action, Consequence)
	& aristotelian_worst_consequence(OtherAction, ConsequenceOtherAction)
	& aristotelian_worse(Consequence, ConsequenceOtherAction)
	& not aristotelian_just(Action).
	
aristotelian_worst_consequence(Action,Consequence):-
	aristotelian_consequence(Action,Consequence)
	& not aristotelian_not_worst_consequence(Action, Consequence).
	
aristotelian_not_worst_consequence(Action, Consequence):-
	aristotelian_consequence(Action, Consequence)
	& aristotelian_consequence(Action, OtherConsequence)
	& aristotelian_less_moral(OtherConsequence, Consequence)
	& not aristotelian_less_moral(Consequence, OtherConsequence).
	
aristotelian_consequence(Action,Action).
	
// Here we adapt these rules in our model
aristotelian_consequence(Action,Consequence):- consequence(Action, Consequence).

aristotelian_less_moral(E1, E2):-
	moral_eval(E1,V1,MV1)
	& moral_eval(E2,V2,MV2)
	& torder_on_moral_valuation(MV2, MV1)
	& not aristotelian_not_less_moral(E1, E2).
	
aristotelian_not_less_moral(E1, E2):-
	moral_eval(E1,V1,MV1)
	& moral_eval(E2,V2,MV2)
	& not torder_on_moral_valuation(MV2, MV1).
	
//*** Ethical Preferences ***//


/*** Ethical preferences ***/
prefEthics("aristotelian","perfectAct").
prefEthics("perfectAct","desireNR").
prefEthics("desireNR","dutyNR").

tPrefEthics(PE1,PE2):- prefEthics(PE1,PE2).
tPrefEthics(PE1,PE2):- prefEthics(PE1,PE3) & tPrefEthics(PE3,PE2).

//*** Ethical Judgment ***//

existBetter(PE1,X):- ethPrinciple(PE1,X) & tPrefEthics(PE2,PE1) & ethPrinciple(PE2,Y).

ethicalJudgment(X,PE):- ethPrinciple(PE,X) & not existBetter(PE,X).
