package marketSim.model;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Date;

import cartago.ArtifactId;

public abstract class AbstractMatchingMethod
{
	protected AbstractAsset asset=null;
	
	protected PrintWriter dataFile=null;
	protected PrintWriter dataDaysFile=null;
	protected int cumulatedVolumeOfDay=0;
	protected float dailyNvar=(float) 0.0;
	
	protected float intradayNvar=(float) 0.0;
	protected float mmIntradayAverage=(float) 0.0;
	protected float mmDoubleIntradayAverage=(float) 0.0;
	protected static int mmIntradayLength=20;
	
	protected AbstractTradingVenue venue;
	protected MarketSimModel model;

	protected ArrayList<AssetDataSet> assetDataSets = new ArrayList<AssetDataSet>();
	protected ArrayList<Integer> volume50D;
	protected Date previousDate=null;
	protected int idOfData=0;

	protected ArrayList<ArtifactId> subscribers = new ArrayList<ArtifactId>();
	
	public AbstractMatchingMethod(MarketSimModel model)
	{
		this.model=model;
		assetDataSets.add(new AssetDataSet(new Date()));
	}
	
	public void setAsset(AbstractAsset asset)
	{
		this.asset=asset;
	}

	/**
	 * @return the venue
	 */
	public AbstractTradingVenue getVenue() {
		return venue;
	}

	/**
	 * @param venue the venue to set
	 */
	public void setVenue(AbstractTradingVenue venue)
	{
		this.venue = venue;
		try
		{
			dataFile=new PrintWriter("./out/data/" + venue.getName() + "-" + asset.getName(),"UTF-8");
			dataDaysFile=new PrintWriter("./out/data/days" + venue.getName() + "-" + asset.getName(),"UTF-8");

			dataFile.println("# Data generated by the simulator");
			dataFile.println("# N°\tDate\tHour\tOpen\tHigh\tLow\tClose\tVolume\tIntensity\tNmlAverage\tDbleLAverage\tBollinger up\tBollinger down");
			dataFile.flush();
		} catch (FileNotFoundException | UnsupportedEncodingException e) {
			e.printStackTrace();
		}
	}

	public void placeOrder(Order order) {
		// TODO Auto-generated method stub
		
	}
	
	public void subscribe(ArtifactId id)
	{
		for(int i=0; i<subscribers.size(); i++)
			if (subscribers.get(i)==id)
				return;
		subscribers.add(id);
	}
	
	public String toString()
	{
		return null;	
	}

	public String getAssetName()
	{
		return asset.getName();
	}

	public void addSubscriber(ArtifactId id)
	{
		if(subscribers.contains(id)) return;
		subscribers.add(id);
		System.out.println("They are now " + subscribers.size() + " subscribers for " + asset.getName());
	}
	
	/**
	 * Diffuse the information about the addition of an order to the list of subscribers
	 * @param order the order added
	 */
	public void diffuseOrderAdditionInformation(Order order)
	{
		for(int i=0; i<subscribers.size(); i++)
			venue.getModel().getArtifact().signalOrderAddition(subscribers.get(i), venue, order);
	}
	
	public void diffuseOrderExecutionInformation(Order newOrder, Order oldOrder, int amount)
	{
		Date t=venue.getModel().getDate();
		for(int i=0; i<subscribers.size(); i++)
			venue.getModel().getArtifact().signalOrderExecution(subscribers.get(i), venue, newOrder, oldOrder, amount, t);
	}
	
	public void diffuseOrderCancelationInformation(Order order)
	{
		Date t=venue.getModel().getDate();
		for(int i=0; i<subscribers.size(); i++)
			venue.getModel().getArtifact().signalOrderCancelation(subscribers.get(i), venue, order);
	}
	
	public void diffuseIndicatorChanges(AssetDataSet a)
	{
		for(int i=0; i<subscribers.size(); i++)
			venue.getModel().getArtifact().signalIndicatorChanges(
					subscribers.get(i),
					a.getDateOfEnd(),
					venue, 
					asset, 
					a.getX_close(),
					a.getV(),
					a.getIntensity(),
					a.getAverage(),
					a.getDoubleLengthAverage(),
					a.getBollingerUp(),
					a.getBollingerDown());
	}

	@SuppressWarnings("deprecation")
	public void onTick(Date date)
	{
		// Initialize
		if(previousDate==null)
		{
			previousDate=date;
			assetDataSets.get(assetDataSets.size()-1).setDateOfBegin(date);
		}
		if(assetDataSets==null)
		{
			assetDataSets=new ArrayList<AssetDataSet>();
		}
		
		// New index block if this is a new day
		if(date.getDay()!=previousDate.getDay())
		{
			dataDaysFile.println(idOfData + " "
					+ (date.getMonth()+1) + "/" + date.getDate() + "/" + (date.getYear()+1900) + " "
					+ cumulatedVolumeOfDay
					);
			dataDaysFile.flush();
			cumulatedVolumeOfDay = 0;
		}

		// Data is computed and printed each minute
		if(date.getMinutes()!=previousDate.getMinutes() 
				&& !(assetDataSets.get(assetDataSets.size()-1).getX_close()==-1.0 && idOfData==0))
		{
			float average, douleLengthAverage, variance;

			// if data is still at default values -> all equals to last close
			if(assetDataSets.size()>=2 && assetDataSets.get(assetDataSets.size()-1).getV()==0)
			{
				float lastClose=assetDataSets.get(assetDataSets.size()-2).getX_close();
				assetDataSets.get(assetDataSets.size()-1).setDefaultValue(lastClose);
			}
			
			// calculation of long-term intraday indicators
			mmIntradayAverage=(float) 0.0;
			mmDoubleIntradayAverage=(float) 0.0;
			intradayNvar=(float) 0.0;
			
			int nbIter = Math.min(mmIntradayLength,assetDataSets.size());
			for(int i=1; i<=nbIter;i++)
			{
				mmIntradayAverage+=assetDataSets.get(assetDataSets.size()-i).getX_close();
			}
			
			average=mmIntradayAverage/nbIter;
			
			for(int i=1; i<=nbIter;i++)
			{
				intradayNvar+=Math.pow(assetDataSets.get(assetDataSets.size()-i).getX_close()-average,2);
			}
			variance=intradayNvar/nbIter;
			
			int nbDblIter = Math.min(2*mmIntradayLength,assetDataSets.size());
			for(int i=1; i<=nbDblIter;i++)
			{
				mmDoubleIntradayAverage+=assetDataSets.get(assetDataSets.size()-i).getX_close();
			}
			douleLengthAverage=mmDoubleIntradayAverage/Math.min(2*mmIntradayLength,idOfData+1);
						
			assetDataSets.get(assetDataSets.size()-1).setAverage(average);
			assetDataSets.get(assetDataSets.size()-1).setVariance(variance);
			assetDataSets.get(assetDataSets.size()-1).setDoubleLengthAverage(douleLengthAverage);
			
			// Intensity calculation
			if(assetDataSets.get(assetDataSets.size()-1).getX_max()!=assetDataSets.get(assetDataSets.size()-1).getX_min())
				assetDataSets.get(assetDataSets.size()-1).setIntensity((
					(2*assetDataSets.get(assetDataSets.size()-1).getX_close())
					-assetDataSets.get(assetDataSets.size()-1).getX_max()
					-assetDataSets.get(assetDataSets.size()-1).getX_min())/((
					assetDataSets.get(assetDataSets.size()-1).getX_max()
					-assetDataSets.get(assetDataSets.size()-1).getX_min())
					*assetDataSets.get(assetDataSets.size()-1).getV()
					)
					);
			else assetDataSets.get(assetDataSets.size()-1).setIntensity((float) 0.0);
			
			assetDataSets.get(assetDataSets.size()-1).setBollingerUp((float) (average+2*Math.sqrt(variance)));
			assetDataSets.get(assetDataSets.size()-1).setBollingerDown((float) (average-2*Math.sqrt(variance)));

			if(assetDataSets.size()!=0)
			{
				assetDataSets.get(assetDataSets.size()-1).setDateOfEnd(date);
				dataFile.println(idOfData + " "
					+ date.getDate() + "/" + (date.getMonth()+1) + "/" + (date.getYear()+1900) + "\t"
					+ date.getHours() + ":" + date.getMinutes() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getX_open() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getX_max() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getX_min() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getX_close() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getV() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getIntensity() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getAverage() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getDoubleLengthAverage() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getBollingerUp() + "\t"
					+ assetDataSets.get(assetDataSets.size()-1).getBollingerDown()
					);
				dataFile.flush();
				diffuseIndicatorChanges(assetDataSets.get(assetDataSets.size()-1));
				cumulatedVolumeOfDay+=assetDataSets.get(assetDataSets.size()-1).getV();
			}
			assetDataSets.add(new AssetDataSet(date));
			idOfData++;
		}
		previousDate=date;
	}
	
	public void registerMatching(int volume, float price)
	{
		assetDataSets.get(assetDataSets.size()-1).putMatching(volume, price);
	}

	public boolean removeOrder(Date date, Side side, AbstractAsset asset, Portfolio portfolio, int amount, int price) {
		return false;
	}
	
	public float getLastClose()
	{
		if(assetDataSets.get(assetDataSets.size()-1).getX_close()!=-1.0)
			return assetDataSets.get(assetDataSets.size()-1).getX_close();
		else return (float) 0.0;
	}
}
